@@ -43,7 +43,6 @@ unsigned long g_window_border_inner_color;
 
 GHashTable* g_clients; // container of all clients
 
-
 // atoms from dwm.c
 // default atoms
 enum { WMProtocols, WMDelete, WMState, WMTakeFocus, WMLast };
@@ -56,8 +55,10 @@ static HSClient* create_client() {
     hc->title = g_string_new("");
     hc->urgent = false;
     hc->fullscreen = false;
+    hc->ewmhfullscreen = false;
     hc->pseudotile = false;
     hc->ewmhrequests = true;
+    hc->ewmhnotify = true;
     return hc;
 }
 
@@ -169,7 +170,6 @@ HSClient* manage_client(Window win) {
 
     // actually manage it
     g_hash_table_insert(g_clients, &(client->window), client);
-    ewmh_add_client(client->window);
     // insert to layout
     if (!client->tag) {
         client->tag = m->tag;
@@ -179,8 +179,8 @@ HSClient* manage_client(Window win) {
     // insert window to the stack
     client->slice = slice_create_client(client);
     stack_insert_slice(client->tag->stack, client->slice);
-    // insert windwo to the tag
-    frame_insert_window_at_index(client->tag->frame, win, changes.tree_index->str);
+    // insert window to the tag
+    frame_insert_window(lookup_frame(client->tag->frame, changes.tree_index->str), win);
     client_update_wm_hints(client);
     if (changes.focus) {
         // give focus to window if wanted
@@ -194,6 +194,9 @@ HSClient* manage_client(Window win) {
     tag_set_flags_dirty();
     client_set_fullscreen(client, changes.fullscreen);
     ewmh_update_window_state(client);
+    // add client after setting the correct tag for the new client
+    // this ensures a panel can read the tag property correctly at this point
+    ewmh_add_client(client->window);
 
     HSMonitor* monitor = find_monitor_with_tag(client->tag);
     if (monitor) {
@@ -284,11 +287,11 @@ void window_unfocus_last() {
 void window_focus(Window window) {
     HSClient* client = get_client_from_window(window);
     assert(client != NULL);
-    // set keyboardfocus
+    // set keyboard focus
     if (!client->neverfocus) {
         XSetInputFocus(g_display, window, RevertToPointerRoot, CurrentTime);
     }
-    client_sendevent(client, g_wmatom[WMTakeFocus]);
+    else client_sendevent(client, g_wmatom[WMTakeFocus]);
 
     if (window != lastfocus) {
         /* FIXME: this is a workaround because window_focus always is called
@@ -319,6 +322,7 @@ void window_focus(Window window) {
     }
     tag_update_focus_layer(get_current_monitor()->tag);
     grab_client_buttons(get_client_from_window(window), true);
+    client_set_urgent(client, false);
 }
 
 void client_setup_border(HSClient* client, bool focused) {
@@ -537,21 +541,34 @@ void window_hide(Window win) {
 }
 
 // heavily inspired by dwm.c
-void client_clear_urgent(HSClient* client) {
-    if (client->urgent) {
-        char winid_str[STRING_BUF_SIZE];
-        snprintf(winid_str, STRING_BUF_SIZE, "0x%lx", client->window);
-        hook_emit_list("urgent", "off", winid_str, NULL);
-        client->urgent = false;
-        XWMHints *wmh;
-        if(!(wmh = XGetWMHints(g_display, client->window)))
-            return;
-        wmh->flags &= ~XUrgencyHint;
-        XSetWMHints(g_display, client->window, wmh);
-        XFree(wmh);
-        // report changes to tags
-        tag_set_flags_dirty();
+void client_set_urgent(HSClient* client, bool state) {
+    if (client->urgent == state) {
+        // nothing to do
+        return;
     }
+
+    char winid_str[STRING_BUF_SIZE];
+    snprintf(winid_str, STRING_BUF_SIZE, "0x%lx", client->window);
+    hook_emit_list("urgent", state ? "on" : "off", winid_str, NULL);
+
+    client->urgent = state;
+
+    client_setup_border(client, client->window == frame_focused_window(g_cur_frame));
+
+    XWMHints *wmh;
+    if(!(wmh = XGetWMHints(g_display, client->window)))
+        return;
+
+    if (state) {
+        wmh->flags |= XUrgencyHint;
+    } else {
+        wmh->flags &= ~XUrgencyHint;
+    }
+
+    XSetWMHints(g_display, client->window, wmh);
+    XFree(wmh);
+    // report changes to tags
+    tag_set_flags_dirty();
 }
 
 // heavily inspired by dwm.c
@@ -624,6 +641,9 @@ void client_set_fullscreen(HSClient* client, bool state) {
     }
 
     client->fullscreen = state;
+    if (client->ewmhnotify) {
+        client->ewmhfullscreen = state;
+    }
     HSStack* stack = client->tag->stack;
     if (state) {
         stack_slice_add_layer(stack, client->slice, LAYER_FULLSCREEN);
@@ -645,9 +665,7 @@ void client_set_pseudotile(HSClient* client, bool state) {
 }
 
 int client_set_property_command(int argc, char** argv) {
-    if (argc < 2) {
-        return HERBST_INVALID_ARGUMENT;
-    }
+    char* action = (argc > 1) ? argv[1] : "toggle";
 
     HSClient* client = get_current_client();
     if (!client) {
@@ -676,7 +694,7 @@ int client_set_property_command(int argc, char** argv) {
     }
 
     // if found, then change it
-    bool state = string_to_bool(argv[1], *(properties[i].value));
+    bool state = string_to_bool(action, *(properties[i].value));
     properties[i].func(client, state);
     return 0;
 }
@@ -770,4 +788,3 @@ bool client_sendevent(HSClient *client, Atom proto) {
     return exists;
 }
 
-
