@@ -3,7 +3,6 @@
  * This software is licensed under the "Simplified BSD License".
  * See LICENSE for details */
 
-
 #ifndef __CLIENTLIST_H_
 #define __CLIENTLIST_H_
 
@@ -27,9 +26,11 @@ typedef struct HSClient {
     GString*    title;  // or also called window title; this is never NULL
     bool        urgent;
     bool        fullscreen;
+    bool        ewmhfullscreen; // ewmh fullscreen state
     bool        pseudotile; // only move client but don't resize (if possible)
     bool        neverfocus; // do not give the focus via XSetInputFocus
     bool        ewmhrequests; // accept ewmh-requests for this client
+    bool        ewmhnotify; // send ewmh-notifications for this client
     int         pid;
     struct HSSlice* slice;
 } HSClient;
@@ -65,7 +66,7 @@ void client_setup_border(HSClient* client, bool focused);
 void client_resize(HSClient* client, XRectangle rect, HSFrame* frame);
 void client_resize_tiling(HSClient* client, XRectangle rect, HSFrame* frame);
 void client_resize_floating(HSClient* client, HSMonitor* m);
-void client_clear_urgent(HSClient* client);
+void client_set_urgent(HSClient* client, bool state);
 void client_update_wm_hints(HSClient* client);
 void client_update_title(HSClient* client);
 void client_raise(HSClient* client);
@@ -89,4 +90,3 @@ unsigned long get_window_border_color(HSClient* client);
 
 #endif
 
-
