@@ -15,6 +15,7 @@
 #include <string.h>
 #include <stdio.h>
 #include <stdint.h>
+#include <stdbool.h>
 
 #include <X11/Xlib.h>
 #include <X11/Xproto.h>
@@ -46,6 +47,7 @@ char* g_netatom_names[NetCOUNT] = {
     [ NetWmState                    ] = "_NET_WM_STATE"                     ,
     [ NetWmWindowOpacity            ] = "_NET_WM_WINDOW_OPACITY"            ,
     [ NetWmStateFullscreen          ] = "_NET_WM_STATE_FULLSCREEN"          ,
+    [ NetWmStateDemandsAttention    ] = "_NET_WM_STATE_DEMANDS_ATTENTION"   ,
     [ NetSupportingWmCheck          ] = "_NET_SUPPORTING_WM_CHECK"          ,
     [ NetWmWindowTypeDesktop        ] = "_NET_WM_WINDOW_TYPE_DESKTOP"       ,
     [ NetWmWindowTypeDock           ] = "_NET_WM_WINDOW_TYPE_DOCK"          ,
@@ -98,9 +100,7 @@ void ewmh_init() {
         XA_WINDOW, 32, PropModeReplace, (unsigned char*)&(g_wm_window), 1);
     XChangeProperty(g_display, g_wm_window, g_netatom[NetSupportingWmCheck],
         XA_WINDOW, 32, PropModeReplace, (unsigned char*)&(g_wm_window), 1);
-    XChangeProperty(g_display, g_wm_window, g_netatom[NetWmName],
-        ATOM("UTF8_STRING"), 8, PropModeReplace,
-        (unsigned char*)WINDOW_MANAGER_NAME, strlen(WINDOW_MANAGER_NAME)+1);
+    ewmh_update_wmname();
 
     /* init atoms that never change */
     int buf[] = { 0, 0 };
@@ -126,6 +126,19 @@ void ewmh_destroy() {
     XDestroyWindow(g_display, g_wm_window);
 }
 
+void ewmh_set_wmname(char* name) {
+    XChangeProperty(g_display, g_wm_window, g_netatom[NetWmName],
+        ATOM("UTF8_STRING"), 8, PropModeReplace,
+        (unsigned char*)name, strlen(name));
+    XChangeProperty(g_display, g_root, g_netatom[NetWmName],
+        ATOM("UTF8_STRING"), 8, PropModeReplace,
+        (unsigned char*)name, strlen(name));
+}
+
+void ewmh_update_wmname() {
+    ewmh_set_wmname(settings_find("wmname")->value.s);
+}
+
 void ewmh_update_client_list() {
     XChangeProperty(g_display, g_root, g_netatom[NetClientList],
         XA_WINDOW, 32, PropModeReplace,
@@ -152,17 +165,48 @@ void ewmh_get_original_client_list(Window** buf, unsigned long *count) {
     *count = g_original_clients_count;
 }
 
-void ewmh_update_client_list_stacking() {
-    int count = monitor_stack_window_count();
-    Window* buf = g_new(Window, count);
+struct ewmhstack {
+    Window* buf;
+    int     count;
+    int     i;  // index of the next free element in buf
+};
+
+static void ewmh_add_tag_stack(HSTag* tag, void* data) {
+    struct ewmhstack* stack = (struct ewmhstack*)data;
+    if (find_monitor_with_tag(tag)) {
+        // do not add tags because they are already added
+        return;
+    }
     int remain;
-    monitor_stack_to_window_buf(buf, count, &remain);
-    array_reverse(buf, count, sizeof(buf[0]));
+    stack_to_window_buf(tag->stack, stack->buf + stack->i,
+                        stack->count - stack->i, true, &remain);
+    if (remain >= 0) {
+        stack->i = stack->count - remain;
+    } else {
+        HSDebug("Warning: not enough space in the ewmh stack\n");
+        stack->i = stack->count;
+    }
+}
+
+void ewmh_update_client_list_stacking() {
+    // First: get the windows in the current stack
+    struct ewmhstack stack;
+    stack.count = g_window_count;
+    stack.buf = g_new(Window, stack.count);
+    int remain;
+    monitor_stack_to_window_buf(stack.buf, stack.count, true, &remain);
+    stack.i = stack.count - remain;
+
+    // Then add all the others at the end
+    tag_foreach(ewmh_add_tag_stack, &stack);
+
+    // reverse stacking order, because ewmh requires bottom to top order
+    array_reverse(stack.buf, stack.count, sizeof(stack.buf[0]));
 
     XChangeProperty(g_display, g_root, g_netatom[NetClientListStacking],
         XA_WINDOW, 32, PropModeReplace,
-        (unsigned char *) buf, count);
-    g_free(buf);
+        (unsigned char *) stack.buf, stack.i);
+    g_free(stack.buf);
 }
 
 void ewmh_add_client(Window win) {
@@ -170,6 +214,7 @@ void ewmh_add_client(Window win) {
     g_windows[g_window_count] = win;
     g_window_count++;
     ewmh_update_client_list();
+    ewmh_update_client_list_stacking();
 }
 
 void ewmh_remove_client(Window win) {
@@ -184,6 +229,7 @@ void ewmh_remove_client(Window win) {
         g_window_count--;
     }
     ewmh_update_client_list();
+    ewmh_update_client_list_stacking();
 }
 
 void ewmh_update_desktops() {
@@ -301,6 +347,8 @@ void ewmh_handle_client_message(XEvent* event) {
             } client_atoms[] = {
                 { NetWmStateFullscreen,
                     client->fullscreen,     client_set_fullscreen },
+                { NetWmStateDemandsAttention,
+                    client->urgent,         client_set_urgent },
             };
 
             /* me->data.l[1] and [2] describe the properties to alter */
@@ -348,7 +396,8 @@ void ewmh_update_window_state(struct HSClient* client) {
         int     atom_index;
         bool    enabled;
     } client_atoms[] = {
-        { NetWmStateFullscreen,         client->fullscreen      },
+        { NetWmStateFullscreen,         client->ewmhfullscreen  },
+        { NetWmStateDemandsAttention,   client->urgent          },
     };
 
     /* find out which flags are set */
@@ -402,3 +451,4 @@ void ewmh_set_window_opacity(Window win, double opacity) {
     XChangeProperty(g_display, win, g_netatom[NetWmWindowOpacity], XA_CARDINAL,
                     32, PropModeReplace, (unsigned char*)&int_opacity, 1);
 }
+
