@@ -36,6 +36,7 @@ enum {
     NetWmWindowOpacity,
     /* window states */
     NetWmStateFullscreen,
+    NetWmStateDemandsAttention,
     /* window types */
     ENUM_WITH_ALIAS(NetWmWindowTypeDesktop, NetWmWindowTypeFIRST),
     NetWmWindowTypeDock,
@@ -68,6 +69,8 @@ void ewmh_update_all();
 
 void ewmh_add_client(Window win);
 void ewmh_remove_client(Window win);
+void ewmh_set_wmname(char* name);
+void ewmh_update_wmname();
 
 void ewmh_update_client_list();
 void ewmh_get_original_client_list(Window** buf, unsigned long *count);
