@@ -42,16 +42,13 @@ int         g_verbose;
 // size for some normal string buffers
 #define STRING_BUF_SIZE 1000
 
-
 #define HSDebug(...) \
     do { \
         if (g_verbose) { \
             fprintf(stderr, "%s: %d: ", __FILE__, __LINE__); \
             fprintf(stderr, __VA_ARGS__); \
         } \
-    } while(0);
+    } while(0)
 
 #endif
 
-
-
