@@ -6,7 +6,6 @@
 #ifndef __HERBST_IPC_PROTOCOL_H_
 #define __HERBST_IPC_PROTOCOL_H_
 
-
 #define HERBST_IPC_CLASS "HERBST_IPC_CLASS"
 //#define HERBST_IPC_READY "HERBST_IPC_READY"
 //#define HERBST_IPC_ATOM  "_HERBST_IPC"
@@ -20,7 +19,6 @@
 // maximum number of hooks to buffer
 #define HERBST_HOOK_PROPERTY_COUNT 10
 
-
 // function exit codes
 enum {
     HERBST_EXIT_SUCCESS = 0,
@@ -31,6 +29,8 @@ enum {
     HERBST_TAG_IN_USE,
     HERBST_FORBIDDEN,
     HERBST_NO_PARAMETER_EXPECTED,
+    HERBST_ENV_UNSET,
+    HERBST_NEED_MORE_ARGS,
 };
 
 #endif
