@@ -80,7 +80,7 @@ unsigned int modifiername2mask(const char* name) {
 /**
  * \brief finds a (any) modifier in mask and returns its name
  *
- * \return  the name as a char string. you must not free it.
+ * \return  the name as a char string. You must not free it.
  */
 char*   modifiermask2name(unsigned int mask) {
     int i;
@@ -92,20 +92,22 @@ char*   modifiermask2name(unsigned int mask) {
     return NULL;
 }
 
-int keybind(int argc, char** argv) {
+int keybind(int argc, char** argv, GString* output) {
     if (argc <= 2) {
-        fprintf(stderr, "keybind: not enough arguments\n");
-        return HERBST_INVALID_ARGUMENT;
+        return HERBST_NEED_MORE_ARGS;
     }
     KeyBinding new_bind;
     // get keycode
     if (!string2key(argv[1], &(new_bind.modifiers), &(new_bind.keysym))) {
+        g_string_append_printf(output,
+            "%s: No such KeySym/modifier\n", argv[0]);
         return HERBST_INVALID_ARGUMENT;
     }
     KeyCode keycode = XKeysymToKeycode(g_display, new_bind.keysym);
     if (!keycode) {
-        fprintf(stderr, "keybind: no keycode for symbol %s\n",
-            XKeysymToString(new_bind.keysym));
+        g_string_append_printf(output,
+            "%s: No keycode for symbol %s\n",
+            argv[0], XKeysymToString(new_bind.keysym));
         return HERBST_INVALID_ARGUMENT;
     }
     // remove existing binding with same keysym/modifiers
@@ -124,7 +126,7 @@ int keybind(int argc, char** argv) {
 
 bool string2modifiers(char* string, unsigned int* modmask) {
     // example strings: "Mod1-space" "Mod4+f" "f"
-    // splitt string at "+-"
+    // split string at "+-"
     char** splitted = g_strsplit_set(string, KEY_COMBI_SEPARATORS, 0);
     // this should give at least one part
     if (NULL == splitted[0]) {
@@ -177,15 +179,18 @@ void handle_key_press(XEvent* ev) {
     GList* element = g_list_find_custom(g_key_binds, &pressed, (GCompareFunc)keysym_equals);
     if (element && element->data) {
         KeyBinding* found = (KeyBinding*)element->data;
+        // duplicate the args in the case this keybinding removes itself
+        char** argv =  argv_duplicate(found->cmd_argc, found->cmd_argv);
+        int argc = found->cmd_argc;
         // call the command
-        call_command_no_output(found->cmd_argc, found->cmd_argv);
+        call_command_no_output(argc, argv);
+        argv_free(argc, argv);
     }
 }
 
-int keyunbind(int argc, char** argv) {
+int keyunbind(int argc, char** argv, GString* output) {
     if (argc <= 1) {
-        fprintf(stderr, "keybind: not enough arguments\n");
-        return HERBST_INVALID_ARGUMENT;
+        return HERBST_NEED_MORE_ARGS;
     }
     // remove all keybinds if wanted
     if (!strcmp(argv[1], "-F") || !strcmp(argv[1], "--all")) {
@@ -196,26 +201,32 @@ int keyunbind(int argc, char** argv) {
     KeySym keysym;
     // get keycode
     if (!string2key(argv[1], &modifiers, &keysym)) {
+        g_string_append_printf(output,
+            "%s: No such KeySym/modifier\n", argv[0]);
         return HERBST_INVALID_ARGUMENT;
     }
-    key_remove_bind_with_keysym(modifiers, keysym);
+    if (key_remove_bind_with_keysym(modifiers, keysym) == false) {
+        g_string_append_printf(output,
+            "%s: Key \"%s\" is not bound\n", argv[0], argv[1]);
+    }
     regrab_keys();
     return 0;
 }
 
-void key_remove_bind_with_keysym(unsigned int modifiers, KeySym keysym){
+bool key_remove_bind_with_keysym(unsigned int modifiers, KeySym keysym){
     KeyBinding bind;
     bind.modifiers = modifiers;
     bind.keysym = keysym;
     // search this keysym in list and remove it
     GList* element = g_list_find_custom(g_key_binds, &bind, (GCompareFunc)keysym_equals);
     if (!element) {
-        return;
+        return false;
     }
     KeyBinding* data = element->data;
     keybinding_free(data);
     g_key_binds = g_list_remove_link(g_key_binds, element);
     g_list_free_1(element);
+    return true;
 }
 
 void regrab_keys() {
@@ -272,8 +283,8 @@ GString* keybinding_to_g_string(KeyBinding* binding) {
         if (!name) {
             break;
         }
-        str = g_string_append(str, name);
-        str = g_string_append_c(str, KEY_COMBI_SEPARATORS[0]);
+        g_string_append(str, name);
+        g_string_append_c(str, KEY_COMBI_SEPARATORS[0]);
         /* remove found mask from mask */
         new_mask = old_mask & ~ modifiername2mask(name);
     }
@@ -284,13 +295,13 @@ GString* keybinding_to_g_string(KeyBinding* binding) {
         g_warning("XKeysymToString failed! using \'?\' instead\n");
         name = "?";
     }
-    str = g_string_append(str, name);
+    g_string_append(str, name);
 
     return str;
 }
 
 struct key_find_context {
-    GString**   output;
+    GString*   output;
     char*       needle;
     size_t      needle_len;
 };
@@ -299,33 +310,33 @@ static void key_find_binds_helper(KeyBinding* b, struct key_find_context* c) {
     GString* name = keybinding_to_g_string(b);
     if (!strncmp(c->needle, name->str, c->needle_len)) {
         /* add to output if key starts with searched needle */
-        *c->output = g_string_append(*c->output, name->str);
-        *c->output = g_string_append_c(*c->output, '\n');
+        g_string_append(c->output, name->str);
+        g_string_append_c(c->output, '\n');
     }
     g_string_free(name, true);
 }
 
-void key_find_binds(char* needle, GString** output) {
+void key_find_binds(char* needle, GString* output) {
     struct key_find_context c = {
         output, needle, strlen(needle)
     };
     g_list_foreach(g_key_binds, (GFunc)key_find_binds_helper, &c);
 }
 
-static void key_list_binds_helper(KeyBinding* b, GString** output) {
+static void key_list_binds_helper(KeyBinding* b, GString* output) {
     // add keybinding
     GString* name = keybinding_to_g_string(b);
-    *output = g_string_append(*output, name->str);
+    g_string_append(output, name->str);
     g_string_free(name, true);
     // add associated command
     for (int i = 0; i < b->cmd_argc; i++) {
-        *output = g_string_append_c(*output, '\t');
-        *output = g_string_append(*output, b->cmd_argv[i]);
+        g_string_append_c(output, '\t');
+        g_string_append(output, b->cmd_argv[i]);
     }
-    *output = g_string_append_c(*output, '\n');
+    g_string_append_c(output, '\n');
 }
 
-int key_list_binds(int argc, char** argv, GString** output) {
+int key_list_binds(int argc, char** argv, GString* output) {
     g_list_foreach(g_key_binds, (GFunc)key_list_binds_helper, output);
     return 0;
 }
