@@ -16,7 +16,7 @@ typedef struct KeyBinding {
     KeySym keysym;
     unsigned int modifiers;
     int     cmd_argc; // number of arguments for command
-    char**  cmd_argv; // arguemnts for command to call
+    char**  cmd_argv; // arguments for command to call
 } KeyBinding;
 
 unsigned int modifiername2mask(const char* name);
@@ -24,15 +24,15 @@ char*   modifiermask2name(unsigned int mask);
 
 bool string2modifiers(char* string, unsigned int* modmask);
 bool string2key(char* string, unsigned int* modmask, KeySym* keysym);
-int keybind(int argc, char** argv);
-int keyunbind(int argc, char** argv); //removes a keybinding
+int keybind(int argc, char** argv, GString* output);
+int keyunbind(int argc, char** argv, GString* output); //removes a keybinding
 void keybinding_free(KeyBinding* binding);
 
-int key_list_binds(int argc, char** argv, GString** output);
-void key_remove_bind_with_keysym(unsigned int modifiers, KeySym sym);
+int key_list_binds(int argc, char** argv, GString* output);
+bool key_remove_bind_with_keysym(unsigned int modifiers, KeySym sym);
 void key_remove_all_binds();
 GString* keybinding_to_g_string(KeyBinding* binding);
-void key_find_binds(char* needle, GString** output);
+void key_find_binds(char* needle, GString* output);
 void regrab_keys();
 void grab_keybind(KeyBinding* binding, void* useless_pointer);
 void update_numlockmask();
