@@ -3,7 +3,6 @@
  * This software is licensed under the "Simplified BSD License".
  * See LICENSE for details */
 
-
 #include "clientlist.h"
 #include "globals.h"
 #include "utils.h"
@@ -13,12 +12,15 @@
 #include "settings.h"
 #include "layout.h"
 #include "stack.h"
+#include "monitor.h"
+
 #include <glib.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <assert.h>
 #include <stdint.h>
+
 #include <X11/Xutil.h>
 #include <X11/Xatom.h>
 #include <X11/Xlib.h>
@@ -32,7 +34,6 @@ int* g_frame_border_width;
 int* g_frame_border_inner_width;
 int* g_always_show_frame;
 int* g_default_frame_layout;
-int* g_focus_follows_shift;
 int* g_frame_bg_transparent;
 int* g_direction_external_only;
 int* g_gapless_grid;
@@ -66,7 +67,6 @@ static void fetch_frame_colors() {
     g_frame_gap = &(settings_find("frame_gap")->value.i);
     g_frame_padding = &(settings_find("frame_padding")->value.i);
     g_window_gap = &(settings_find("window_gap")->value.i);
-    g_focus_follows_shift = &(settings_find("focus_follows_shift")->value.i);
     g_frame_border_width = &(settings_find("frame_border_width")->value.i);
     g_frame_border_inner_width = &(settings_find("frame_border_inner_width")->value.i);
     g_always_show_frame = &(settings_find("always_show_frame")->value.i);
@@ -97,7 +97,7 @@ static void fetch_frame_colors() {
         g_warning("too few characters in setting tree_style\n");
         // ensure that it is long enough
         char* argv[] = { "set", "tree_style", "01234567" };
-        settings_set_command(LENGTH(argv), argv);
+        settings_set_command(LENGTH(argv), argv, NULL);
     }
 }
 
@@ -123,7 +123,7 @@ HSFrame* frame_create_empty(HSFrame* parent, HSTag* parenttag) {
     frame->content.clients.layout = *g_default_frame_layout;
     frame->parent = parent;
     frame->tag = parent ? parent->tag : parenttag;
-    // set window atributes
+    // set window attributes
     XSetWindowAttributes at;
     at.background_pixel  = getcolor("red");
     at.background_pixmap = ParentRelative;
@@ -179,30 +179,30 @@ void frame_insert_window(HSFrame* frame, Window window) {
     }
 }
 
-void frame_insert_window_at_index(HSFrame* frame, Window window, char* index) {
-    if (!index || index[0] == '\0') {
-        frame_insert_window(frame, window);
-    }
-    else if (frame->type == TYPE_CLIENTS) {
-        frame_insert_window(frame, window);
-    } else { /* frame->type == TYPE_FRAMES */
-        HSLayout* layout = &frame->content.layout;
-        HSFrame* frame;
-        switch (index[0]) {
-            case '0': frame = layout->a; break;
-            case '1': frame = layout->b; break;
-            /* opposite selection */
-            case '/': frame = (layout->selection == 0)
-                                ?  layout->b
-                                :  layout->a; break;
-            /* else just follow selection */
-            case '.':
-            default:  frame = (layout->selection == 0)
-                                ?  layout->a
-                                : layout->b; break;
-        }
-        frame_insert_window_at_index(frame, window, index + 1);
+HSFrame* lookup_frame(HSFrame* root, char *index) {
+    if (index == NULL || index[0] == '\0') return root;
+    if (root->type == TYPE_CLIENTS) return root;
+
+    HSFrame* new_root;
+    char *new_index = index + 1;
+    HSLayout* layout = &root->content.layout;
+
+    switch (index[0]) {
+        case '0': new_root = layout->a; break;
+        case '1': new_root = layout->b; break;
+        /* opposite selection */
+        case '/': new_root = (layout->selection == 0)
+                            ?  layout->b
+                            :  layout->a;
+                  break;
+        /* else just follow selection */
+        case '@': new_index = index;
+        case '.':
+        default:  new_root = (layout->selection == 0)
+                            ?  layout->a
+                            : layout->b; break;
     }
+    return lookup_frame(new_root, new_index);
 }
 
 bool frame_remove_window(HSFrame* frame, Window window) {
@@ -264,9 +264,9 @@ void frame_destroy(HSFrame* frame, Window** buf, size_t* count) {
     g_free(frame);
 }
 
-void dump_frame_tree(HSFrame* frame, GString** output) {
+void dump_frame_tree(HSFrame* frame, GString* output) {
     if (frame->type == TYPE_CLIENTS) {
-        g_string_append_printf(*output, "%cclients%c%s:%d",
+        g_string_append_printf(output, "%cclients%c%s:%d",
             LAYOUT_DUMP_BRACKETS[0],
             LAYOUT_DUMP_WHITESPACES[0],
             g_layout_names[frame->content.clients.layout],
@@ -274,14 +274,14 @@ void dump_frame_tree(HSFrame* frame, GString** output) {
         Window* buf = frame->content.clients.buf;
         size_t i, count = frame->content.clients.count;
         for (i = 0; i < count; i++) {
-            g_string_append_printf(*output, "%c0x%lx",
+            g_string_append_printf(output, "%c0x%lx",
                 LAYOUT_DUMP_WHITESPACES[0],
                 buf[i]);
         }
-        g_string_append_c(*output, LAYOUT_DUMP_BRACKETS[1]);
+        g_string_append_c(output, LAYOUT_DUMP_BRACKETS[1]);
     } else {
         /* type == TYPE_FRAMES */
-        g_string_append_printf(*output, "%csplit%c%s%c%lf%c%d%c",
+        g_string_append_printf(output, "%csplit%c%s%c%lf%c%d%c",
             LAYOUT_DUMP_BRACKETS[0],
             LAYOUT_DUMP_WHITESPACES[0],
             g_align_names[frame->content.layout.align],
@@ -291,17 +291,17 @@ void dump_frame_tree(HSFrame* frame, GString** output) {
             frame->content.layout.selection,
             LAYOUT_DUMP_WHITESPACES[0]);
         dump_frame_tree(frame->content.layout.a, output);
-        g_string_append_c(*output, LAYOUT_DUMP_WHITESPACES[0]);
+        g_string_append_c(output, LAYOUT_DUMP_WHITESPACES[0]);
         dump_frame_tree(frame->content.layout.b, output);
-        g_string_append_c(*output, LAYOUT_DUMP_BRACKETS[1]);
+        g_string_append_c(output, LAYOUT_DUMP_BRACKETS[1]);
     }
 }
 
-char* load_frame_tree(HSFrame* frame, char* description, GString** errormsg) {
+char* load_frame_tree(HSFrame* frame, char* description, GString* errormsg) {
     // find next (
     description = strchr(description, LAYOUT_DUMP_BRACKETS[0]);
     if (!description) {
-        g_string_append_printf(*errormsg, "missing %c\n",
+        g_string_append_printf(errormsg, "Missing %c\n",
             LAYOUT_DUMP_BRACKETS[0]);
         return NULL;
     }
@@ -327,10 +327,12 @@ char* load_frame_tree(HSFrame* frame, char* description, GString** errormsg) {
     // jump over args substring
     description += args_len;
     if (!*description) {
+        g_string_append_printf(errormsg, "Missing %c or arguments\n", LAYOUT_DUMP_BRACKETS[1]);
         return NULL;
     }
     description += strspn(description, LAYOUT_DUMP_WHITESPACES);
     if (!*description) {
+        g_string_append_printf(errormsg, "Missing %c or arguments\n", LAYOUT_DUMP_BRACKETS[1]);
         return NULL;
     }
 
@@ -343,16 +345,16 @@ char* load_frame_tree(HSFrame* frame, char* description, GString** errormsg) {
 #define SEP LAYOUT_DUMP_SEPARATOR_STR
         if (3 != sscanf(args, "%[^"SEP"]"SEP"%lf"SEP"%d",
             align_name, &fraction_double, &selection)) {
-            g_string_append_printf(*errormsg,
-                    "cannot parse frame args \"%s\"\n", args);
+            g_string_append_printf(errormsg,
+                "Can not parse frame args \"%s\"\n", args);
             return NULL;
         }
 #undef SEP
         int align = find_align_by_name(align_name);
         g_free(align_name);
         if (align < 0) {
-            g_string_append_printf(*errormsg,
-                    "invalid align name in args \"%s\"\n", args);
+            g_string_append_printf(errormsg,
+                "Invalid alignment name in args \"%s\"\n", args);
             return NULL;
         }
         selection = !!selection; // CLAMP it to [0;1]
@@ -366,8 +368,8 @@ char* load_frame_tree(HSFrame* frame, char* description, GString** errormsg) {
         } else {
             frame_split(frame, align, fraction);
             if (frame->type != TYPE_FRAMES) {
-                g_string_append_printf(*errormsg,
-                    "cannot split frame");
+                g_string_append_printf(errormsg,
+                    "Can not split frame\n");
                 return NULL;
             }
         }
@@ -387,16 +389,16 @@ char* load_frame_tree(HSFrame* frame, char* description, GString** errormsg) {
 #define SEP LAYOUT_DUMP_SEPARATOR_STR
         if (2 != sscanf(args, "%[^"SEP"]"SEP"%d",
             layout_name, &selection)) {
-            g_string_append_printf(*errormsg,
-                    "cannot parse frame args \"%s\"\n", args);
+            g_string_append_printf(errormsg,
+                "Can not parse frame args \"%s\"\n", args);
             return NULL;
         }
 #undef SEP
         int layout = find_layout_by_name(layout_name);
         g_free(layout_name);
         if (layout < 0) {
-            g_string_append_printf(*errormsg,
-                    "cannot parse layout from args \"%s\"\n", args);
+            g_string_append_printf(errormsg,
+                "Can not parse layout from args \"%s\"\n", args);
             return NULL;
         }
 
@@ -420,8 +422,8 @@ char* load_frame_tree(HSFrame* frame, char* description, GString** errormsg) {
             frame->type = TYPE_CLIENTS;
             frame->content.clients.buf = buf;
             frame->content.clients.count = count;
-            frame->content.clients.selection = 0; // only some sane defauts
-            frame->content.clients.layout = 0; // only some sane defauts
+            frame->content.clients.selection = 0; // only some sane defaults
+            frame->content.clients.layout = 0; // only some sane defaults
         }
 
         // bring child wins
@@ -432,8 +434,8 @@ char* load_frame_tree(HSFrame* frame, char* description, GString** errormsg) {
         while (*description != LAYOUT_DUMP_BRACKETS[1]) {
             Window win;
             if (1 != sscanf(description, "0x%lx\n", &win)) {
-                g_string_append_printf(*errormsg,
-                        "cannot parse window id from \"%s\"\n", description);
+                g_string_append_printf(errormsg,
+                    "Can not parse window id from \"%s\"\n", description);
                 return NULL;
             }
             // jump over window id and over whitespaces
@@ -450,7 +452,7 @@ char* load_frame_tree(HSFrame* frame, char* description, GString** errormsg) {
             // remove window from old tag
             HSMonitor* clientmonitor = find_monitor_with_tag(client->tag);
             if (!frame_remove_window(client->tag->frame, win)) {
-                g_warning("window %lx wasnot found on tag %s\n",
+                g_warning("window %lx was not found on tag %s\n",
                     win, client->tag->name->str);
             }
             if (clientmonitor) {
@@ -486,7 +488,7 @@ char* load_frame_tree(HSFrame* frame, char* description, GString** errormsg) {
     if (*description == LAYOUT_DUMP_BRACKETS[1]) {
         description++;
     } else {
-        g_string_append_printf(*errormsg, "warning: missing closing bracket %c\n", LAYOUT_DUMP_BRACKETS[1]);
+        g_string_append_printf(errormsg, "warning: missing closing bracket %c\n", LAYOUT_DUMP_BRACKETS[1]);
     }
     // and over whitespaces
     description += strspn(description, LAYOUT_DUMP_WHITESPACES);
@@ -522,23 +524,23 @@ HSFrame* get_toplevel_frame(HSFrame* frame) {
     return frame;
 }
 
-static void frame_append_caption(HSTree tree, GString** output) {
+static void frame_append_caption(HSTree tree, GString* output) {
     HSFrame* frame = (HSFrame*) tree;
     if (frame->type == TYPE_CLIENTS) {
         // list of clients
-        g_string_append_printf(*output, "%s:",
+        g_string_append_printf(output, "%s:",
             g_layout_names[frame->content.clients.layout]);
         Window* buf = frame->content.clients.buf;
         size_t i, count = frame->content.clients.count;
         for (i = 0; i < count; i++) {
-            g_string_append_printf(*output, " 0x%lx", buf[i]);
+            g_string_append_printf(output, " 0x%lx", buf[i]);
         }
         if (g_cur_frame == frame) {
-            *output = g_string_append(*output, " [FOCUS]");
+            g_string_append(output, " [FOCUS]");
         }
     } else {
         /* type == TYPE_FRAMES */
-        g_string_append_printf(*output, "%s %d%% selection=%d",
+        g_string_append_printf(output, "%s %d%% selection=%d",
             g_layout_names[frame->content.layout.align],
             frame->content.layout.fraction * 100 / FRACTION_UNIT,
             frame->content.layout.selection);
@@ -565,13 +567,13 @@ static HSTreeInterface frame_nth_child(HSTree tree, size_t idx) {
     return intf;
 }
 
-void print_tag_tree(HSTag* tag, GString** output) {
+void print_frame_tree(HSFrame* frame, GString* output) {
     HSTreeInterface frameintf = {
         .child_count    = frame_child_count,
         .nth_child      = frame_nth_child,
         .append_caption = frame_append_caption,
         .destructor     = NULL,
-        .data           = (HSTree) tag->frame,
+        .data           = (HSTree) frame,
     };
     tree_print_to(&frameintf, output);
 }
@@ -596,7 +598,8 @@ void frame_apply_floating_layout(HSFrame* frame, HSMonitor* m) {
     }
 }
 
-int frame_current_cycle_client_layout(int argc, char** argv) {
+int frame_current_cycle_client_layout(int argc, char** argv, GString* output) {
+    char* cmd_name = argv[0]; // save this before shifting
     int delta = 1;
     if (argc >= 2) {
         delta = atoi(argv[1]);
@@ -616,6 +619,8 @@ int frame_current_cycle_client_layout(int argc, char** argv) {
         idx %= argc;
         layout_index = find_layout_by_name(argv[idx]);
         if (layout_index < 0) {
+            g_string_append_printf(output,
+                "%s: Invalid layout name \"%s\"\n", cmd_name, argv[idx]);
             return HERBST_INVALID_ARGUMENT;
         }
     } else {
@@ -630,15 +635,15 @@ int frame_current_cycle_client_layout(int argc, char** argv) {
     return 0;
 }
 
-int frame_current_set_client_layout(int argc, char** argv) {
+int frame_current_set_client_layout(int argc, char** argv, GString* output) {
     int layout = 0;
     if (argc <= 1) {
-        fprintf(stderr, "set_layout: not enough arguments\n");
-        return HERBST_INVALID_ARGUMENT;
+        return HERBST_NEED_MORE_ARGS;
     }
     layout = find_layout_by_name(argv[1]);
     if (layout < 0) {
-        HSDebug("set_layout: invalid layout name \"%s\"\n", argv[1]);
+        g_string_append_printf(output,
+            "%s: Invalid layout name \"%s\"\n", argv[0], argv[1]);
         return HERBST_INVALID_ARGUMENT;
     }
     if (g_cur_frame && g_cur_frame->type == TYPE_CLIENTS) {
@@ -674,7 +679,7 @@ void frame_apply_client_layout_linear(HSFrame* frame, XRectangle rect, bool vert
     }
     for (int i = 0; i < count; i++) {
         HSClient* client = get_client_from_window(buf[i]);
-        // add the space, if count doesnot divide frameheight without remainder
+        // add the space, if count does not divide frameheight without remainder
         cur.height += (i == count-1) ? last_step_y : 0;
         cur.width += (i == count-1) ? last_step_x : 0;
         client_setup_border(client, (g_cur_frame == frame) && (i == selection));
@@ -870,7 +875,7 @@ static void frame_update_frame_window_visibility_helper(HSFrame* frame) {
 }
 
 void frame_update_frame_window_visibility(HSFrame* frame) {
-    frame_do_recursive(frame, frame_update_frame_window_visibility_helper, 0);
+    frame_do_recursive(frame, frame_update_frame_window_visibility_helper, 2);
 }
 
 HSFrame* frame_current_selection() {
@@ -885,11 +890,21 @@ HSFrame* frame_current_selection() {
     return frame;
 }
 
-int frame_current_bring(int argc, char** argv) {
-    HSClient* client;
+int frame_current_bring(int argc, char** argv, GString* output) {
+    HSClient* client = NULL;
 
-    string_to_client((argc > 1) ? argv[1] : "", &client);
+    if (argc < 2) {
+        return HERBST_NEED_MORE_ARGS;
+    }
+    string_to_client(argv[1], &client);
     if (!client) {
+        g_string_append_printf(output,
+            "%s: Could not find client", argv[0]);
+        if (argc > 1) {
+            g_string_append_printf(output, " \"%s\".\n", argv[1]);
+        } else {
+            g_string_append(output, ".\n");
+        }
         return HERBST_INVALID_ARGUMENT;
     }
     tag_move_client(client, get_current_monitor()->tag);
@@ -902,7 +917,7 @@ int frame_current_set_selection(int argc, char** argv) {
     if (argc >= 2) {
         index = atoi(argv[1]);
     } else {
-        return HERBST_INVALID_ARGUMENT;
+        return HERBST_NEED_MORE_ARGS;
     }
     // find current selection
     HSFrame* frame = frame_current_selection();
@@ -1038,7 +1053,7 @@ int cycle_all_command(int argc, char** argv) {
             int oldselection = frame->content.layout.selection;
             if (oldselection == direction) {
                 // if we already reached the end,
-                // i.e. if we cannot go in the disired direction
+                // i.e. if we cannot go in the desired direction
                 // then wrap around
                 frame->content.layout.selection = other_direction;
                 frame = frame->content.layout.a;
@@ -1141,15 +1156,21 @@ void frame_split(HSFrame* frame, int align, int fraction) {
     frame->content.layout.fraction = fraction;
 }
 
-int frame_split_command(int argc, char** argv) {
+int frame_split_command(int argc, char** argv, GString* output) {
     // usage: split h|v FRACTION
     if (argc < 3) {
-        return HERBST_INVALID_ARGUMENT;
+        return HERBST_NEED_MORE_ARGS;
     }
     int align = ALIGN_VERTICAL;
     if (argv[1][0] == 'h') {
         align = ALIGN_HORIZONTAL;
-    } // else: layout ist vertical
+    } else if (argv[1][0] == 'v') {
+        align = ALIGN_VERTICAL;
+    } else {
+        g_string_append_printf(output,
+            "%s: Invalid alignment \"%s\"\n", argv[0], argv[1]);
+        return HERBST_INVALID_ARGUMENT;
+    }
     int fraction = FRACTION_UNIT* CLAMP(atof(argv[2]),
                                         0.0 + FRAME_MIN_FRACTION,
                                         1.0 - FRAME_MIN_FRACTION);
@@ -1163,10 +1184,10 @@ int frame_split_command(int argc, char** argv) {
     return 0;
 }
 
-int frame_change_fraction_command(int argc, char** argv) {
+int frame_change_fraction_command(int argc, char** argv, GString* output) {
     // usage: fraction DIRECTION DELTA
     if (argc < 3) {
-        return HERBST_INVALID_ARGUMENT;
+        return HERBST_NEED_MORE_ARGS;
     }
     char direction = argv[1][0];
     double delta_double = atof(argv[2]);
@@ -1180,7 +1201,10 @@ int frame_change_fraction_command(int argc, char** argv) {
         case 'r':   break;
         case 'u':   delta *= -1; break;
         case 'd':   break;
-        default:    return HERBST_INVALID_ARGUMENT;
+        default:
+            g_string_append_printf(output,
+                "%s: Invalid direction \"%s\"\n", argv[0], argv[1]);
+            return HERBST_INVALID_ARGUMENT;
     }
     HSFrame* neighbour = frame_neighbour(g_cur_frame, direction);
     if (!neighbour) {
@@ -1194,8 +1218,9 @@ int frame_change_fraction_command(int argc, char** argv) {
         }
         neighbour = frame_neighbour(g_cur_frame, direction);
         if (!neighbour) {
-            // nothing to do
-            return 0;
+            g_string_append_printf(output,
+                "%s: No neighbour found\n", argv[0]);
+            return HERBST_FORBIDDEN;
         }
     }
     HSFrame* parent = neighbour->parent;
@@ -1313,9 +1338,9 @@ int frame_inner_neighbour_index(HSFrame* frame, char direction) {
     return index;
 }
 
-int frame_focus_command(int argc, char** argv) {
+int frame_focus_command(int argc, char** argv, GString* output) {
     // usage: focus [-e|-i] left|right|up|down
-    if (argc < 2) return HERBST_INVALID_ARGUMENT;
+    if (argc < 2) return HERBST_NEED_MORE_ARGS;
     if (!g_cur_frame) {
         fprintf(stderr, "warning: no frame is selected\n");
         return HERBST_UNKNOWN_ERROR;
@@ -1348,14 +1373,18 @@ int frame_focus_command(int argc, char** argv) {
             // change focus if possible
             frame_focus_recursive(parent);
             monitor_apply_layout(get_current_monitor());
+        } else {
+            g_string_append_printf(output,
+                "%s: No neighbour found\n", argv[0]);
+            return HERBST_FORBIDDEN;
         }
     }
     return 0;
 }
 
-int frame_move_window_command(int argc, char** argv) {
+int frame_move_window_command(int argc, char** argv, GString* output) {
     // usage: move left|right|up|down
-    if (argc < 2) return HERBST_INVALID_ARGUMENT;
+    if (argc < 2) return HERBST_NEED_MORE_ARGS;
     if (!g_cur_frame) {
         fprintf(stderr, "warning: no frame is selected\n");
         return HERBST_UNKNOWN_ERROR;
@@ -1380,9 +1409,7 @@ int frame_move_window_command(int argc, char** argv) {
         buf[selection] = buf[index];
         buf[index] = tmp;
 
-        if (*g_focus_follows_shift) {
-            g_cur_frame->content.clients.selection = index;
-        }
+        g_cur_frame->content.clients.selection = index;
         frame_focus_recursive(g_cur_frame);
         monitor_apply_layout(get_current_monitor());
     } else {
@@ -1392,30 +1419,32 @@ int frame_move_window_command(int argc, char** argv) {
             // move window to neighbour
             frame_remove_window(g_cur_frame, win);
             frame_insert_window(neighbour, win);
-            if (*g_focus_follows_shift) {
-                // change selection in parrent
-                HSFrame* parent = neighbour->parent;
-                assert(parent);
-                parent->content.layout.selection = ! parent->content.layout.selection;
-                frame_focus_recursive(parent);
-                // focus right window in frame
-                HSFrame* frame = g_cur_frame;
-                assert(frame);
-                int i;
-                Window* buf = frame->content.clients.buf;
-                size_t count = frame->content.clients.count;
-                for (i = 0; i < count; i++) {
-                    if (buf[i] == win) {
-                        frame->content.clients.selection = i;
-                        window_focus(buf[i]);
-                        break;
-                    }
+
+            // change selection in parent
+            HSFrame* parent = neighbour->parent;
+            assert(parent);
+            parent->content.layout.selection = ! parent->content.layout.selection;
+            frame_focus_recursive(parent);
+            // focus right window in frame
+            HSFrame* frame = g_cur_frame;
+            assert(frame);
+            int i;
+            Window* buf = frame->content.clients.buf;
+            size_t count = frame->content.clients.count;
+            for (i = 0; i < count; i++) {
+                if (buf[i] == win) {
+                    frame->content.clients.selection = i;
+                    window_focus(buf[i]);
+                    break;
                 }
-            } else {
-                frame_focus_recursive(g_cur_frame);
             }
+
             // layout was changed, so update it
             monitor_apply_layout(get_current_monitor());
+        } else {
+            g_string_append_printf(output,
+                "%s: No neighbour found\n", argv[0]);
+            return HERBST_FORBIDDEN;
         }
     }
     return 0;
@@ -1481,8 +1510,8 @@ bool frame_focus_window(HSFrame* frame, Window win) {
 }
 
 // focus a window
-// switch_tag tells, wether to switch tag to focus to window
-// switch_monitor tells, wether to switch monitor to focus to window
+// switch_tag tells, whether to switch tag to focus to window
+// switch_monitor tells, whether to switch monitor to focus to window
 // returns if window was focused or not
 bool focus_window(Window win, bool switch_tag, bool switch_monitor) {
     HSClient* client = get_client_from_window(win);
@@ -1513,10 +1542,12 @@ bool focus_window(Window win, bool switch_tag, bool switch_monitor) {
             assert(cur_mon == monitor);
         }
     }
+    monitors_lock();
     monitor_set_tag(cur_mon, tag);
     cur_mon = get_current_monitor();
     if (cur_mon->tag != tag) {
         // could not set tag on monitor
+        monitors_unlock();
         return false;
     }
     // now the right tag is visible
@@ -1524,6 +1555,7 @@ bool focus_window(Window win, bool switch_tag, bool switch_monitor) {
     bool found = frame_focus_window(tag->frame, win);
     frame_focus_recursive(tag->frame);
     monitor_apply_layout(cur_mon);
+    monitors_unlock();
     return found;
 }
 
@@ -1596,7 +1628,7 @@ static void frame_show_clients(HSFrame* frame) {
 }
 
 void frame_show_recursive(HSFrame* frame) {
-    // first show parents, then childrend => order = 0
+    // first show parents, then children => order = 0
     frame_do_recursive(frame, frame_show_clients, 2);
 }
 
@@ -1696,7 +1728,7 @@ void frame_set_visible(HSFrame* frame, bool visible) {
 }
 
 // executes action for each client within frame and its subframes
-// if action fails (i.e. returns something != 0), then it abborts with this code
+// if action fails (i.e. returns something != 0), then it aborts with this code
 int frame_foreach_client(HSFrame* frame, ClientAction action, void* data) {
     int status;
     if (frame->type == TYPE_FRAMES) {
@@ -1725,7 +1757,6 @@ int frame_foreach_client(HSFrame* frame, ClientAction action, void* data) {
     return 0;
 }
 
-
 void frame_update_border(Window window, unsigned long color) {
     if (*g_frame_border_inner_width > 0 && *g_frame_border_inner_width < *g_frame_border_width) {
         set_window_double_border(g_display, window, *g_frame_border_inner_width, g_frame_border_inner_color, color);
@@ -1733,3 +1764,24 @@ void frame_update_border(Window window, unsigned long color) {
         XSetWindowBorder(g_display, window, color);
     }
 }
+
+int frame_focus_edge(int argc, char** argv, GString* output) {
+    // Puts the focus to the edge in the specified direction
+    char* args[] = { "" };
+    monitors_lock_command(LENGTH(args), args);
+    while (0 == frame_focus_command(argc,argv,output))
+        ;
+    monitors_unlock_command(LENGTH(args), args);
+    return 0;
+}
+
+int frame_move_window_edge(int argc, char** argv, GString* output) {
+    // Moves a window to the edge in the specified direction
+    char* args[] = { "" };
+    monitors_lock_command(LENGTH(args), args);
+    while (0 == frame_move_window_command(argc,argv,output))
+        ;
+    monitors_unlock_command(LENGTH(args), args);
+    return 0;
+}
+
