@@ -20,9 +20,10 @@
 
 #define TAG_SET_FLAG(tag, flag) \
     ((tag)->flags |= (flag))
+
 enum {
     TAG_FLAG_URGENT = 0x01, // is there a urgent window?
-    TAG_FLAG_USED   = 0x02, // the oposit of empty
+    TAG_FLAG_USED   = 0x02, // the opposite of empty
 };
 
 enum {
@@ -97,7 +98,7 @@ void layout_destroy();
 // for frames
 HSFrame* frame_create_empty(HSFrame* parent, HSTag* parenttag);
 void frame_insert_window(HSFrame* frame, Window window);
-void frame_insert_window_at_index(HSFrame* frame, Window window, char* index);
+HSFrame* lookup_frame(HSFrame* root, char* path);
 HSFrame* frame_current_selection();
 // removes window from a frame/subframes
 // returns true, if window was found. else: false
@@ -107,8 +108,8 @@ bool frame_remove_window(HSFrame* frame, Window window);
 // YOU have to g_free the resulting window-buf
 void frame_destroy(HSFrame* frame, Window** buf, size_t* count);
 void frame_split(HSFrame* frame, int align, int fraction);
-int frame_split_command(int argc, char** argv);
-int frame_change_fraction_command(int argc, char** argv);
+int frame_split_command(int argc, char** argv, GString* output);
+int frame_change_fraction_command(int argc, char** argv, GString* output);
 
 void frame_apply_layout(HSFrame* frame, XRectangle rect);
 void frame_apply_floating_layout(HSFrame* frame, struct HSMonitor* m);
@@ -116,17 +117,16 @@ void frame_update_frame_window_visibility(HSFrame* frame);
 void reset_frame_colors();
 HSFrame* get_toplevel_frame(HSFrame* frame);
 
-void print_tag_tree(HSTag* tag, GString** output);
-void print_frame_tree(HSFrame* frame, char* indent, char* rootprefix, GString** output);
-void dump_frame_tree(HSFrame* frame, GString** output);
+void print_frame_tree(HSFrame* frame, GString* output);
+void dump_frame_tree(HSFrame* frame, GString* output);
 // create apply a described layout to a frame and its subframes
-// returns pointer to string that wasnot parsed yet
+// returns pointer to string that was not parsed yet
 // or NULL on an error
-char* load_frame_tree(HSFrame* frame, char* description, GString** errormsg);
+char* load_frame_tree(HSFrame* frame, char* description, GString* errormsg);
 int find_layout_by_name(char* name);
 int find_align_by_name(char* name);
 
-int frame_current_bring(int argc, char** argv);
+int frame_current_bring(int argc, char** argv, GString* output);
 int frame_current_set_selection(int argc, char** argv);
 int frame_current_cycle_selection(int argc, char** argv);
 int cycle_all_command(int argc, char** argv);
@@ -137,7 +137,7 @@ void frame_unfocus(); // unfocus currently focused window
 // returns the neighbour or NULL if there is no one
 HSFrame* frame_neighbour(HSFrame* frame, char direction);
 int frame_inner_neighbour_index(HSFrame* frame, char direction);
-int frame_focus_command(int argc, char** argv);
+int frame_focus_command(int argc, char** argv, GString* output);
 
 // follow selection to leave and focus this frame
 int frame_focus_recursive(HSFrame* frame);
@@ -152,8 +152,8 @@ int frame_foreach_client(HSFrame* frame, ClientAction action, void* data);
 void frame_apply_client_layout_linear(HSFrame* frame, XRectangle rect, bool vertical);
 void frame_apply_client_layout_horizontal(HSFrame* frame, XRectangle rect);
 void frame_apply_client_layout_vertical(HSFrame* frame, XRectangle rect);
-int frame_current_cycle_client_layout(int argc, char** argv);
-int frame_current_set_client_layout(int argc, char** argv);
+int frame_current_cycle_client_layout(int argc, char** argv, GString* output);
+int frame_current_set_client_layout(int argc, char** argv, GString* output);
 int frame_split_count_to_root(HSFrame* frame, int align);
 
 // returns the Window that is focused
@@ -162,13 +162,15 @@ Window frame_focused_window(HSFrame* frame);
 bool frame_focus_window(HSFrame* frame, Window win);
 bool focus_window(Window win, bool switch_tag, bool switch_monitor);
 // moves a window to an other frame
-int frame_move_window_command(int argc, char** argv);
+int frame_move_window_command(int argc, char** argv, GString* output);
 /// removes the current frame
 int frame_remove_command(int argc, char** argv);
 int close_or_remove_command(int argc, char** argv);
 void frame_set_visible(HSFrame* frame, bool visible);
 void frame_update_border(Window window, unsigned long color);
 
+int frame_focus_edge(int argc, char** argv, GString* output);
+int frame_move_window_edge(int argc, char** argv, GString* output);
+
 #endif
 
-
