@@ -124,6 +124,7 @@ CommandBinding g_commands[] = {
     CMD_BIND(             "add",            tag_add_command),
     CMD_BIND(             "use",            monitor_set_tag_command),
     CMD_BIND(             "use_index",      monitor_set_tag_by_index_command),
+    CMD_BIND(             "use_previous",   monitor_set_previous_tag_command),
     CMD_BIND(             "jumpto",         jumpto_command),
     CMD_BIND(             "floating",       tag_set_floating_command),
     CMD_BIND_NO_OUTPUT(   "fullscreen",     client_set_property_command),
@@ -264,7 +265,6 @@ int load_command(int argc, char** argv, GString* output) {
 
 int print_tag_status_command(int argc, char** argv, GString* output) {
     HSMonitor* monitor;
-    int monitor_index = g_cur_monitor;
     if (argc >= 2) {
         monitor = string_to_monitor(argv[1]);
     } else {
@@ -287,7 +287,7 @@ int print_tag_status_command(int argc, char** argv, GString* output) {
         HSMonitor *tag_monitor = find_monitor_with_tag(tag);
         if (tag_monitor == monitor) {
             c = '+';
-            if (monitor_index == g_cur_monitor) {
+            if (monitor == get_current_monitor()) {
                 c = '#';
             }
         } else if (tag_monitor) {
