@@ -388,6 +388,7 @@ HSMonitor* add_monitor(XRectangle rect, HSTag* tag, char* name) {
     HSMonitor* m = g_new0(HSMonitor, 1);
     m->rect = rect;
     m->tag = tag;
+    m->tag_previous = tag;
     m->name = (name ? g_string_new(name) : NULL);
     m->mouse.x = 0;
     m->mouse.y = 0;
@@ -448,8 +449,8 @@ int add_monitor_command(int argc, char** argv, GString* output) {
         }
     }
     HSMonitor* monitor = add_monitor(rect, tag, name);
-    frame_show_recursive(tag->frame);
     monitor_apply_layout(monitor);
+    frame_show_recursive(tag->frame);
     emit_tag_changed(tag, g_monitors->len - 1);
     return 0;
 }
@@ -504,6 +505,9 @@ int remove_monitor(int index) {
         g_cur_monitor--;
         // if selection has changed, then relayout focused monitor
         monitor_apply_layout(get_current_monitor());
+        // also announce the new selection
+        ewmh_update_current_desktop();
+        emit_tag_changed(get_current_monitor()->tag, g_cur_monitor);
     }
     return 0;
 }
@@ -734,6 +738,8 @@ int monitor_set_tag(HSMonitor* monitor, HSTag* tag) {
         return 0;
     }
     HSTag* old_tag = monitor->tag;
+    // save old tag
+    monitor->tag_previous = old_tag;
     // 1. show new tag
     monitor->tag = tag;
     // first reset focus and arrange windows
@@ -808,6 +814,26 @@ int monitor_set_tag_by_index_command(int argc, char** argv, GString* output) {
     return ret;
 }
 
+int monitor_set_previous_tag_command(int argc, char** argv, GString* output) {
+    if (argc < 1) {
+        return HERBST_NEED_MORE_ARGS;
+    }
+    HSMonitor* monitor = get_current_monitor();
+    HSTag*  tag = monitor->tag_previous;
+    if (monitor && tag) {
+        int ret = monitor_set_tag(monitor, tag);
+        if (ret != 0) {
+            g_string_append_printf(output,
+                    "%s: Could not change tag (maybe monitor is locked?)\n", argv[0]);
+        }
+        return ret;
+    } else {
+        g_string_append_printf(output,
+                "%s: Invalid monitor or tag\n", argv[0]);
+        return HERBST_INVALID_ARGUMENT;
+    }
+}
+
 int monitor_focus_command(int argc, char** argv, GString* output) {
     if (argc < 2) {
         return HERBST_NEED_MORE_ARGS;
@@ -1175,3 +1201,12 @@ int shift_to_monitor(int argc, char** argv, GString* output) {
     return 0;
 }
 
+void all_monitors_replace_previous_tag(HSTag *old, HSTag *new) {
+    int i;
+    for (i = 0; i < g_monitors->len; i++) {
+        HSMonitor* m = monitor_with_index(i);
+        if (m->tag_previous == old) {
+            m->tag_previous = new;
+        }
+    }
+}
