@@ -20,6 +20,7 @@ struct HSStack;
 
 typedef struct HSMonitor {
     struct HSTag*      tag;    // currently viewed tag
+    struct HSTag*      tag_previous;    // previously viewed tag
     struct HSSlice*    slice;  // slice in the monitor stack
     GString*    name;
     int         pad_up;
@@ -78,6 +79,7 @@ int monitor_set_tag(HSMonitor* monitor, struct HSTag* tag);
 int monitor_set_pad_command(int argc, char** argv, GString* output);
 int monitor_set_tag_command(int argc, char** argv, GString* output);
 int monitor_set_tag_by_index_command(int argc, char** argv, GString* output);
+int monitor_set_previous_tag_command(int argc, char** argv, GString* output);
 void monitors_lock();
 void monitors_unlock();
 int monitors_lock_command(int argc, char** argv);
@@ -88,6 +90,7 @@ int monitor_unlock_tag_command(int argc, char** argv, GString* output);
 void monitor_apply_layout(HSMonitor* monitor);
 void all_monitors_apply_layout();
 void ensure_monitors_are_available();
+void all_monitors_replace_previous_tag(struct HSTag* old, struct HSTag* new);
 
 void monitor_restack(HSMonitor* monitor);
 int monitor_stack_window_count(bool only_clients);
