@@ -8,6 +8,7 @@
 
 #include <X11/Xlib.h>
 #include <stdbool.h>
+#include <glib.h>
 
 // various snap-flags
 enum SnapFlags {
@@ -20,7 +21,7 @@ enum SnapFlags {
         SNAP_EDGE_TOP | SNAP_EDGE_BOTTOM | SNAP_EDGE_LEFT | SNAP_EDGE_RIGHT,
 };
 
-// foreward declarations
+// forward declarations
 struct HSClient;
 struct HSTag;
 
@@ -39,7 +40,7 @@ int mouse_binding_equals(MouseBinding* a, MouseBinding* b);
 
 void mouse_bind_function(unsigned int modifiers, unsigned int button,
                          MouseFunction function);
-int mouse_bind_command(int argc, char** argv);
+int mouse_bind_command(int argc, char** argv, GString* output);
 int mouse_unbind_all();
 MouseBinding* mouse_binding_find(unsigned int modifiers, unsigned int button);
 
@@ -66,6 +67,5 @@ void mouse_function_move(XMotionEvent* me);
 void mouse_function_resize(XMotionEvent* me);
 void mouse_function_zoom(XMotionEvent* me);
 
-
 #endif
 
