@@ -9,13 +9,14 @@
 #include "ewmh.h"
 #include "clientlist.h"
 #include "ipc-protocol.h"
+#include "hook.h"
 
 #include <glib.h>
+#include "glib-backports.h"
 #include <string.h>
 #include <stdio.h>
 #include <sys/types.h>
 
-
 /// TYPES ///
 
 typedef struct {
@@ -59,6 +60,8 @@ DECLARE_CONSEQUENCE(consequence_pseudotile);
 DECLARE_CONSEQUENCE(consequence_fullscreen);
 DECLARE_CONSEQUENCE(consequence_switchtag);
 DECLARE_CONSEQUENCE(consequence_ewmhrequests);
+DECLARE_CONSEQUENCE(consequence_ewmhnotify);
+DECLARE_CONSEQUENCE(consequence_hook);
 
 /// GLOBALS ///
 
@@ -84,6 +87,8 @@ static HSConsequenceType g_consequence_types[] = {
     { "pseudotile",     consequence_pseudotile      },
     { "fullscreen",     consequence_fullscreen      },
     { "ewmhrequests",   consequence_ewmhrequests    },
+    { "ewmhnotify",     consequence_ewmhnotify      },
+    { "hook",           consequence_hook            },
 };
 
 GQueue g_rules = G_QUEUE_INIT; // a list of HSRule* elements
@@ -112,10 +117,11 @@ int find_condition_type(char* name) {
     return -1;
 }
 
-HSCondition* condition_create(int type, char op, char* value) {
+HSCondition* condition_create(int type, char op, char* value, GString* output) {
     HSCondition cond;
     if (op != '=' && type == g_maxage_type) {
-        fprintf(stderr, "condition maxage only supports the = operator\n");
+        g_string_append_printf(output,
+            "rule: Condition maxage only supports the = operator\n");
         return NULL;
     }
     switch (op) {
@@ -123,7 +129,8 @@ HSCondition* condition_create(int type, char op, char* value) {
             if (type == g_maxage_type) {
                 cond.value_type = CONDITION_VALUE_TYPE_INTEGER;
                 if (1 != sscanf(value, "%d", &cond.value.integer)) {
-                    fprintf(stderr, "cannot integer from \"%s\"\n", value);
+                    g_string_append_printf(output,
+                        "rule: Can not integer from \"%s\"\n", value);
                     return NULL;
                 }
             } else {
@@ -138,16 +145,16 @@ HSCondition* condition_create(int type, char op, char* value) {
             if (status != 0) {
                 char buf[ERROR_STRING_BUF_SIZE];
                 regerror(status, &cond.value.exp, buf, ERROR_STRING_BUF_SIZE);
-                fprintf(stderr, "Cannot parse value \"%s\"", value);
-                fprintf(stderr, "from condition \"%s\": ",
-                        g_condition_types[type].name);
-                fprintf(stderr, "\"%s\"\n", buf);
+                g_string_append_printf(output,
+                    "rule: Can not parse value \"%s\" from condition \"%s\": \"%s\"\n",
+                    value, g_condition_types[type].name, buf);
                 return NULL;
             }
             break;
 
         default:
-            fprintf(stderr, "unknown rule condition operation \"%c\"\n", op);
+            g_string_append_printf(output,
+                "rule: Unknown rule condition operation \"%c\"\n", op);
             return NULL;
             break;
     }
@@ -192,7 +199,7 @@ int find_consequence_type(char* name) {
     return -1;
 }
 
-HSConsequence* consequence_create(int type, char op, char* value) {
+HSConsequence* consequence_create(int type, char op, char* value, GString* output) {
     HSConsequence cons;
     switch (op) {
         case '=':
@@ -201,7 +208,8 @@ HSConsequence* consequence_create(int type, char op, char* value) {
             break;
 
         default:
-            fprintf(stderr, "unknown rule consequence operation \"%c\"\n", op);
+            g_string_append_printf(output,
+                "rule: Unknown rule consequence operation \"%c\"\n", op);
             return NULL;
             break;
     }
@@ -286,9 +294,12 @@ static void rule_add_consequence(HSRule* rule, HSConsequence* cons) {
 }
 
 
-int rule_add_command(int argc, char** argv) {
+int rule_add_command(int argc, char** argv, GString* output) {
     // usage: rule COND=VAL ... then
 
+    if (argc < 2) {
+        return HERBST_NEED_MORE_ARGS;
+    }
     // temporary data structures
     HSRule* rule = rule_create();
     HSCondition* cond;
@@ -303,6 +314,7 @@ int rule_add_command(int argc, char** argv) {
         { "once",   &rule->once },
     };
 
+    // parse rule incrementally. always maintain a correct rule in rule
     while (SHIFT(argc, argv)) {
         char* name;
         char* value;
@@ -327,7 +339,7 @@ int rule_add_command(int argc, char** argv) {
         }
 
         else if (consorcond && (type = find_condition_type(name)) >= 0) {
-            cond = condition_create(type, op, value);
+            cond = condition_create(type, op, value, output);
             if (!cond) {
                 rule_destroy(rule);
                 return HERBST_INVALID_ARGUMENT;
@@ -338,7 +350,7 @@ int rule_add_command(int argc, char** argv) {
         }
 
         else if (consorcond && (type = find_consequence_type(name)) >= 0) {
-            cons = consequence_create(type, op, value);
+            cons = consequence_create(type, op, value, output);
             if (!cons) {
                 rule_destroy(rule);
                 return HERBST_INVALID_ARGUMENT;
@@ -347,7 +359,10 @@ int rule_add_command(int argc, char** argv) {
         }
 
         else {
-            fprintf(stderr, "rule: unknown argument \"%s\"\n", *argv);
+            // need to hardcode "rule:" here because args are shifted
+            g_string_append_printf(output,
+                "rule: Unknown argument \"%s\"\n", *argv);
+            rule_destroy(rule);
             return HERBST_INVALID_ARGUMENT;
         }
     }
@@ -356,9 +371,9 @@ int rule_add_command(int argc, char** argv) {
     return 0;
 }
 
-int rule_remove_command(int argc, char** argv) {
+int rule_remove_command(int argc, char** argv, GString* output) {
     if (argc < 2) {
-        return HERBST_INVALID_ARGUMENT;
+        return HERBST_NEED_MORE_ARGS;
     }
 
     if (!strcmp(argv[1], "--all") || !strcmp(argv[1], "-F")) {
@@ -368,6 +383,8 @@ int rule_remove_command(int argc, char** argv) {
         return 0;
     }
 
+    g_string_append_printf(output,
+        "%s: This command must be used with --all/-F\n", argv[0]);
     return HERBST_INVALID_ARGUMENT;
 }
 
@@ -416,7 +433,7 @@ void rules_apply(HSClient* client, HSClientChanges* changes) {
 
             if (!matches && !rule->conditions[i]->negated
                 && rule->conditions[i]->condition_type == g_maxage_type) {
-                // if if not negated maxage doesnot match anymore
+                // if if not negated maxage does not match anymore
                 // then it will never match again in the future
                 rule_expired = true;
             }
@@ -501,7 +518,6 @@ bool condition_title(HSCondition* rule, HSClient* client) {
     return condition_string(rule, client->title->str);
 }
 
-
 bool condition_pid(HSCondition* rule, HSClient* client) {
     if (client->pid < 0) {
         return false;
@@ -562,7 +578,7 @@ bool condition_windowtype(HSCondition* rule, HSClient* client) {
         // try to find the window type
         if (wintype == g_netatom[i]) {
             // if found, then treat the window type as a string value,
-            // which is registred in g_netatom_names[]
+            // which is registered in g_netatom_names[]
             return condition_string(rule, g_netatom_names[i]);
         }
     }
@@ -585,7 +601,7 @@ bool condition_windowrole(HSCondition* rule, HSClient* client) {
 void consequence_tag(HSConsequence* cons,
                      HSClient* client, HSClientChanges* changes) {
     if (changes->tag_name) {
-        changes->tag_name = g_string_assign(changes->tag_name, cons->value.str);
+        g_string_assign(changes->tag_name, cons->value.str);
     } else {
         changes->tag_name = g_string_new(cons->value.str);
     }
@@ -603,7 +619,7 @@ void consequence_manage(HSConsequence* cons, HSClient* client,
 
 void consequence_index(HSConsequence* cons, HSClient* client,
                                HSClientChanges* changes) {
-    changes->tree_index = g_string_assign(changes->tree_index, cons->value.str);
+    g_string_assign(changes->tree_index, cons->value.str);
 }
 
 void consequence_pseudotile(HSConsequence* cons, HSClient* client,
@@ -628,4 +644,13 @@ void consequence_ewmhrequests(HSConsequence* cons, HSClient* client,
     client->ewmhrequests = string_to_bool(cons->value.str, client->ewmhrequests);
 }
 
+void consequence_ewmhnotify(HSConsequence* cons, HSClient* client,
+                            HSClientChanges* changes) {
+    client->ewmhnotify = string_to_bool(cons->value.str, client->ewmhnotify);
+}
 
+void consequence_hook(HSConsequence* cons, HSClient* client,
+                            HSClientChanges* changes) {
+    char* hook_str[] = { "rule" , cons->value.str };
+    hook_emit(LENGTH(hook_str), hook_str);
+}
