@@ -3,12 +3,13 @@
  * This software is licensed under the "Simplified BSD License".
  * See LICENSE for details */
 
-
+#include "globals.h"
 #include "settings.h"
 #include "clientlist.h"
 #include "layout.h"
 #include "ipc-protocol.h"
 #include "utils.h"
+#include "ewmh.h"
 
 #include <glib.h>
 #include <string.h>
@@ -34,6 +35,7 @@
 #define CL_COLORS reset_client_colors
 #define LOCK_CHANGED monitors_lock_changed
 #define FOCUS_LAYER tag_update_each_focus_layer
+#define WMNAME ewmh_update_wmname
 
 // default settings:
 SettingsPair g_settings[] = {
@@ -42,6 +44,7 @@ SettingsPair g_settings[] = {
     SET_INT(    "window_gap",                      0,           RELAYOUT    ),
     SET_INT(    "snap_distance",                   10,          NULL        ),
     SET_INT(    "snap_gap",                        5,           NULL        ),
+    SET_INT(    "mouse_recenter_gap",              0,           NULL        ),
     SET_STRING( "frame_border_active_color",       "red",       FR_COLORS   ),
     SET_STRING( "frame_border_normal_color",       "blue",      FR_COLORS   ),
     SET_STRING( "frame_border_inner_color",        "black",     FR_COLORS   ),
@@ -61,12 +64,11 @@ SettingsPair g_settings[] = {
     SET_INT(    "always_show_frame",               0,           RELAYOUT    ),
     SET_INT(    "default_direction_external_only", 0,           NULL        ),
     SET_INT(    "default_frame_layout",            0,           FR_COLORS   ),
-    SET_INT(    "focus_follows_shift",             1,           NULL        ),
     SET_INT(    "focus_follows_mouse",             0,           NULL        ),
     SET_INT(    "focus_stealing_prevention",       1,           NULL        ),
     SET_INT(    "swap_monitors_to_get_tag",        1,           NULL        ),
     SET_INT(    "raise_on_focus",                  0,           NULL        ),
-    SET_INT(    "raise_on_focus_temporarily",      1,           FOCUS_LAYER ),
+    SET_INT(    "raise_on_focus_temporarily",      0,           FOCUS_LAYER ),
     SET_INT(    "raise_on_click",                  1,           NULL        ),
     SET_INT(    "gapless_grid",                    1,           RELAYOUT    ),
     SET_INT(    "smart_frame_surroundings",        0,           RELAYOUT    ),
@@ -74,6 +76,7 @@ SettingsPair g_settings[] = {
     SET_INT(    "monitors_locked",                 0,           LOCK_CHANGED),
     SET_INT(    "auto_detect_monitors",            0,           NULL        ),
     SET_STRING( "tree_style",                      "*| +`--.",  FR_COLORS   ),
+    SET_STRING( "wmname",                  WINDOW_MANAGER_NAME, WMNAME      ),
 };
 
 int             g_initial_monitors_locked = 0;
@@ -106,20 +109,28 @@ void settings_destroy() {
     }
 }
 
-
 SettingsPair* settings_find(char* name) {
     return STATIC_TABLE_FIND_STR(SettingsPair, g_settings, name, name);
 }
 
-int settings_set_command(int argc, char** argv) {
+int settings_set_command(int argc, char** argv, GString* output) {
     if (argc < 3) {
-        return HERBST_INVALID_ARGUMENT;
+        return HERBST_NEED_MORE_ARGS;
     }
     SettingsPair* pair = settings_find(argv[1]);
     if (!pair) {
+        if (output != NULL) {
+            g_string_append_printf(output,
+                "%s: Setting \"%s\" not found\n", argv[0], argv[1]);
+        }
         return HERBST_SETTING_NOT_FOUND;
     }
-    return settings_set(pair, argv[2]);
+    int ret = settings_set(pair, argv[2]);
+    if (ret == HERBST_INVALID_ARGUMENT) {
+        g_string_append_printf(output,
+            "%s: Invalid value for setting \"%s\"\n", argv[0], argv[1]);
+    }
+    return ret;
 }
 
 int settings_set(SettingsPair* pair, char* value) {
@@ -143,37 +154,40 @@ int settings_set(SettingsPair* pair, char* value) {
         g_free(pair->value.s);
         pair->value.s = g_strdup(value);
     }
-    // on successfull change, call callback
+    // on successful change, call callback
     if (pair->on_change) {
         pair->on_change();
     }
     return 0;
 }
 
-int settings_get(int argc, char** argv, GString** output) {
+int settings_get(int argc, char** argv, GString* output) {
     if (argc < 2) {
-        return HERBST_INVALID_ARGUMENT;
+        return HERBST_NEED_MORE_ARGS;
     }
     SettingsPair* pair = settings_find(argv[1]);
     if (!pair) {
+        g_string_append_printf(output,
+            "%s: Setting \"%s\" not found\n", argv[0], argv[1]);
         return HERBST_SETTING_NOT_FOUND;
     }
     if (pair->type == HS_Int) {
-        g_string_printf(*output, "%d", pair->value.i);
+        g_string_append_printf(output, "%d", pair->value.i);
     } else { // pair->type == HS_String
-        *output = g_string_assign(*output, pair->value.s);
+        g_string_append(output, pair->value.s);
     }
     return 0;
 }
 
-
 // toggle integer-like values
-int settings_toggle(int argc, char** argv) {
+int settings_toggle(int argc, char** argv, GString* output) {
     if (argc < 2) {
-        return HERBST_INVALID_ARGUMENT;
+        return HERBST_NEED_MORE_ARGS;
     }
     SettingsPair* pair = settings_find(argv[1]);
     if (!pair) {
+        g_string_append_printf(output,
+            "%s: Setting \"%s\" not found\n", argv[0], argv[1]);
         return HERBST_SETTING_NOT_FOUND;
     }
     if (pair->type == HS_Int) {
@@ -187,9 +201,11 @@ int settings_toggle(int argc, char** argv) {
         }
     } else {
         // only toggle numbers
+        g_string_append_printf(output,
+            "%s: Only numbers can be toggled\n", argv[0]);
         return HERBST_INVALID_ARGUMENT;
     }
-    // on successfull change, call callback
+    // on successful change, call callback
     if (pair->on_change) {
         pair->on_change();
     }
@@ -205,12 +221,16 @@ bool memberequals_settingspair(void* pmember, void* needle) {
     }
 }
 
-int settings_cycle_value(int argc, char** argv) {
+int settings_cycle_value(int argc, char** argv, GString* output) {
     if (argc < 3) {
-        return HERBST_INVALID_ARGUMENT;
+        return HERBST_NEED_MORE_ARGS;
     }
+    char* cmd_name = argv[0];
+    char* setting_name = argv[1]; // save this before shifting
     SettingsPair* pair = settings_find(argv[1]);
     if (!pair) {
+        g_string_append_printf(output,
+            "%s: Setting \"%s\" not found\n", argv[0], argv[1]);
         return HERBST_SETTING_NOT_FOUND;
     }
     (void)SHIFT(argc, argv);
@@ -218,6 +238,11 @@ int settings_cycle_value(int argc, char** argv) {
     char** pcurrent = table_find(argv, sizeof(*argv), argc, 0,
                                  memberequals_settingspair, pair);
     int i = pcurrent ? ((INDEX_OF(argv, pcurrent) + 1) % argc) : 0;
-    return settings_set(pair, argv[i]);
+    int ret = settings_set(pair, argv[i]);
+    if (ret == HERBST_INVALID_ARGUMENT) {
+        g_string_append_printf(output,
+            "%s: Invalid value for setting \"%s\"\n", cmd_name, setting_name);
+    }
+    return ret;
 }
 
