@@ -16,8 +16,7 @@
 #include <X11/Xutil.h>
 #include <X11/Xatom.h>
 #include <glib.h>
-
-
+#include "glib-backports.h"
 
 #include <time.h>
 #include <sys/time.h>
@@ -27,10 +26,8 @@
 #include <mach/mach.h>
 #endif
 
-
 char*   g_tree_style; /* the one from layout.c */
 
-
 time_t get_monotonic_timestamp() {
     struct timespec ts;
 #if defined(__MACH__) && ! defined(CLOCK_REALTIME) // OS X does not have clock_gettime, use clock_get_time
@@ -80,8 +77,8 @@ GString* window_property_to_g_string(Display* dpy, Window window, Atom atom) {
         return NULL;
     }
     // convert text property to a gstring
-    result = g_string_new("");
-    if (prop.encoding == XA_STRING || prop.encoding == ATOM("UTF8_STRING")) {
+    if (prop.encoding == XA_STRING
+        || prop.encoding == XInternAtom(dpy, "UTF8_STRING", False)) {
         result = g_string_new((char*)prop.value);
     } else {
         if (XmbTextPropertyToTextList(dpy, &prop, &list, &n) >= Success
@@ -170,7 +167,7 @@ char** argv_duplicate(int argc, char** argv) {
     return new_argv;
 }
 
-// frees all entrys in argument-vector and then the vector itself
+// frees all entries in argument-vector and then the vector itself
 void argv_free(int argc, char** argv) {
     int i;
     for (i = 0; i < argc; i++) {
@@ -272,7 +269,7 @@ void array_reverse(void* void_buf, size_t elems, size_t size) {
 /**
  * \brief   tells if the string needle is identical to the string *pmember
  */
-bool  memberequals_string(void* pmember, void* needle) {
+bool memberequals_string(void* pmember, void* needle) {
     return !strcmp(*(char**)pmember, (char*)needle);
 }
 
@@ -284,7 +281,7 @@ bool memberequals_int(void* pmember, void* needle) {
 }
 
 /**
- * \brief   finds a element in a table (i.e. array of structs)
+ * \brief   finds an element in a table (i.e. array of structs)
  *
  *          it consecutively searches from the beginning of the table for a
  *          table element whose member is equal to needle. It passes a pointer
@@ -385,36 +382,35 @@ void set_window_double_border(Display *dpy, Window win, int ibw,
 }
 
 static void subtree_print_to(HSTreeInterface* intface, char* indent,
-                          char* rootprefix, GString** output) {
+                          char* rootprefix, GString* output) {
     HSTree root = intface->data;
     size_t child_count = intface->child_count(root);
     if (child_count == 0) {
-        g_string_append(*output, rootprefix);
-        *output = g_string_append_unichar(*output,
-            UTF8_STRING_AT(g_tree_style, 5));
-        *output = g_string_append_c(*output, ' ');
+        g_string_append(output, rootprefix);
+        g_string_append_unichar(output, UTF8_STRING_AT(g_tree_style, 6));
+        g_string_append_unichar(output, UTF8_STRING_AT(g_tree_style, 5));
+        g_string_append_c(output, ' ');
         // append caption
         intface->append_caption(root, output);
-        *output = g_string_append(*output, "\n");
+        g_string_append(output, "\n");
     } else {
-        g_string_append_printf(*output, "%s", rootprefix);
-        *output = g_string_append_unichar(*output,
-            UTF8_STRING_AT(g_tree_style, 6));
-        *output = g_string_append_unichar(*output,
-            UTF8_STRING_AT(g_tree_style, 7));
-        // apend caption
+        g_string_append_printf(output, "%s", rootprefix);
+        g_string_append_unichar(output, UTF8_STRING_AT(g_tree_style, 6));
+        g_string_append_unichar(output, UTF8_STRING_AT(g_tree_style, 7));
+        // append caption
+        g_string_append_c(output, ' ');
         intface->append_caption(root, output);
-        *output = g_string_append_c(*output, '\n');
-        // apend children
+        g_string_append_c(output, '\n');
+        // append children
         GString* child_indent = g_string_new("");
         GString* child_prefix = g_string_new("");
         for (size_t i = 0; i < child_count; i++) {
             bool last = (i == child_count - 1);
             g_string_printf(child_indent, "%s ", indent);
-            child_indent = g_string_append_unichar(child_indent,
+            g_string_append_unichar(child_indent,
                 UTF8_STRING_AT(g_tree_style, last ? 2 : 1));
             g_string_printf(child_prefix, "%s ", indent);
-            child_prefix = g_string_append_unichar(child_prefix,
+            g_string_append_unichar(child_prefix,
                 UTF8_STRING_AT(g_tree_style, last ? 4 : 3));
             HSTreeInterface child = intface->nth_child(root, i);
             subtree_print_to(&child, child_indent->str,
@@ -429,11 +425,16 @@ static void subtree_print_to(HSTreeInterface* intface, char* indent,
     }
 }
 
-void tree_print_to(HSTreeInterface* intface, GString** output) {
+void tree_print_to(HSTreeInterface* intface, GString* output) {
     GString* root_indicator = g_string_new("");
-    root_indicator = g_string_append_unichar(root_indicator,
-            UTF8_STRING_AT(g_tree_style, 0));
+    g_string_append_unichar(root_indicator, UTF8_STRING_AT(g_tree_style, 0));
     subtree_print_to(intface, " ", root_indicator->str, output);
     g_string_free(root_indicator, true);
 }
 
+int min(int a, int b) {
+    if (a < b)
+        return a;
+    return b;
+}
+
