@@ -16,7 +16,6 @@
 #define SHIFT(ARGC, ARGV) (--(ARGC) && ++(ARGV))
 #define MOD(X, N) ((((X) % (signed)(N)) + (signed)(N)) % (signed)(N))
 
-
 /// print a printf-like message to stderr and exit
 void die(const char *errstr, ...);
 
@@ -35,12 +34,12 @@ struct HSTreeInterface;
 typedef struct HSTreeInterface {
     struct HSTreeInterface  (*nth_child)(HSTree root, size_t idx);
     size_t                  (*child_count)(HSTree root);
-    void                    (*append_caption)(HSTree root, GString** output);
+    void                    (*append_caption)(HSTree root, GString* output);
     HSTree                  data;
     void                    (*destructor)(HSTree data); /* how to free the data tree */
 } HSTreeInterface;
 
-void tree_print_to(HSTreeInterface* intface, GString** output);
+void tree_print_to(HSTreeInterface* intface, GString* output);
 
 
 bool is_herbstluft_window(Display* dpy, Window window);
@@ -58,7 +57,7 @@ time_t get_monotonic_timestamp();
 
 // duplicates an argument-vector
 char** argv_duplicate(int argc, char** argv);
-// frees all entrys in argument-vector and then the vector itself
+// frees all entries in argument-vector and then the vector itself
 void argv_free(int argc, char** argv);
 
 XRectangle parse_rectangle(char* string);
@@ -69,6 +68,7 @@ void g_queue_remove_element(GQueue* queue, GList* elem);
 int array_find(void* buf, size_t elems, size_t size, void* needle);
 void array_reverse(void* buf, size_t elems, size_t size);
 
+int min(int a, int b);
 
 // utils for tables
 typedef bool (*MemberEquals)(void* pmember, void* needle);
@@ -109,4 +109,3 @@ void set_window_double_border(Display *dpy, Window win, int ibw,
 
 #endif
 
-
